const io = require("socket.io");
const AuthService = require("./AuthService");
const UpService = require("./UpService");
const adqGetDashboardFiliais = require("../model/adqGetDashboardFiliais");
const parametrizacoesModel = require("../model/parametrizacoes");

const paginacao = require("../functions/Paginacao");

const UpdaterSocket = require("./UpdaterSocket");
const SocketMessages = require("./SocketMessages");
const MonitoringSocket = require("./MonitoringSocket");

class SocketService {
    getUpdaterSocket() {
        return UpdaterSocket;
    }
    getMonitoringSocket() {
        return MonitoringSocket;
    }
    connectedUnit() {
        this.getRoom("ASSISTANT_MASTER_ROOM").emit("login", {
            text: "UNIDADE CONECTADA",
            error: false,
            message: {
                data: this.slavelist,
            },
        });
    }
    getSocketIo() {
        return this.io;
    }
    getRoom(roomName) {
        return this.getSocketIo().to(roomName);
    }
    async start(http) {
        this.slavelist = await this.getDashboardFiliais();
        this.intervals = []; // Registrar is intervals
        this.portalSocketId = null;

        this.monitoringResult = {};
        this.io = io(http, { cors: "*" });
        MonitoringSocket.setSocketIo(this.io);
        MonitoringSocket.setSlaveList(this.slavelist);
        UpdaterSocket.setSocketIo(this.io);

        this.io.on("connection", (socket) => {
            try {
                const { log: info, agent, version, cdfilial } = this.getSocketHeaderInfo(socket);
                console.log(`OnConnection (${info}): ${socket.id}`);
                let roomName = "";
                switch (agent) {
                    case "updater":
                        roomName = "ASSISTANT_UPDATER_ROOM";
                        UpdaterSocket.validateVersionCompatibility(socket, version, info);
                        break;
                    case "client":
                        roomName = "ASSISTANT_CLIENT_ROOM";
                        break;
                    default:
                        roomName = "ASSISTANT_MASTER_ROOM";
                }
                socket.join(roomName);

                if(agent === 'client') {
                    let curUnit = this.slavelist.find(unit => unit.id == cdfilial);
                    if(curUnit) {
                        curUnit.situacao = "Ativo";
                        this.connectedUnit();
                    }
                }

                /**
                 * Atualiza informações do Dashboard, inclusive o ID do socket e registra a sala
                 */
                socket.on("join", (room) => {
                    this.slavelist.filter((filial) => {
                        if (filial.id === room.id) {
                            Object.assign(filial, room);
                        }
                        return filial;
                    });
                    socket.join(room.room);
                });

                /** Garante que o portal está conectado */
                socket.on("ping", (data) => {
                    const clientId = data.id ? data.id : data;
                    let { agent } = this.getSocketHeaderInfo(socket);
                    if (data.filial && agent == 'client') {
                        /** se for loja, atualiza os dados e avise o portal */
                        this.slavelist.forEach((filial) => {
                            if (filial.id === data.filial) {
                                Object.assign(filial, {
                                    socketId: data.id,
                                    situacao: "Ativo",
                                });
                            }
                        });
                        /** Atualiza o Portal */
                        this.connectedUnit();
                    }

                    /** Confere as conexões */
                    MonitoringSocket.getClients().then((clients) => {
                        clients.forEach((portais) => {
                            if (portais.id === clientId) {
                                portais.time = new Date().getTime();
                            }
                        });
                    });
                });

                /** Remove conexões inativas */
                MonitoringSocket.createIntervalValidateInactiveUser(
                    socket.id,
                    this.io
                );
                /** Garante que o portal está conectado */

                socket.on("login", (data) => {
                    this.login(socket, data);
                });

                /**
                 * Exportação de vendas
                 */
                socket.on("iniciaExportacao", (data) => {
                    try {
                        data.payload = JSON.parse(data.payload);
                        UpService.setInsereServidor(
                            data.CDFILIAL,
                            data.CDCAIXA,
                            data.CDLOJA,
                            data.payload
                        );
                    } catch (e) {}
                });

                /**
                 * Importação
                 */
                socket.on("importar", (data) => {
                    console.log("importar", data);
                    let target = this.slavelist[socket.email][data.CDFILIAL];
                    this.io.to(target.id).emit("command", {
                        type: "import",
                        data: { dataUrl: "/API/getStructure" },
                    });
                });

                /**
                 * Desconet do socket
                 */
                socket.on("disconnect", async () => {
                    /** Limpa os Intervals */
                    MonitoringSocket.removeIntervalValidateInactiveUser(
                        socket.id
                    );

                    /** Limpa as Conexões (Portais e Lojas) */
                    let removed = await MonitoringSocket.removeClientBySocketId(
                        socket.id
                    );
                    let agent = this.getSocketHeaderInfo(socket).agent.toLowerCase();
                    let isClient = agent == "client";
                    let isUpdater = agent == "updater";
                    if (removed) {
                        let subsidiaryIndex = this.slavelist.findIndex(
                            (item) => {
                                return item.socketId === socket.id;
                            }
                        );
                        if (subsidiaryIndex == -1) {
                            subsidiaryIndex = this.slavelist.findIndex(
                                (item) =>
                                    socket.CDFILIAL &&
                                    item.codigo == socket.CDFILIAL
                            );
                        }
                        let curUnit = this.slavelist[subsidiaryIndex];
                        if (curUnit) {
                            curUnit.situacao = "Inativa";
                            if(isUpdater) {
                                curUnit.CONNECTION_STATUS = false;
                            } else if(isClient) {
                                MonitoringSocket.updateStatusClientIndicators(
                                    curUnit
                                );
                            }
                            let roomName = `ASSISTANT_MASTER_ROOM`;
                            this.getRoom(roomName).emit("login", {
                                text: "ASSISTANT_CLIENT_DISCONNECTED",
                                error: false,
                                message: {
                                    data: this.slavelist,
                                },
                            });
                        }
                    } else {
                        let removed =
                            await UpdaterSocket.removeClientBySocketId(
                                socket.id
                            );
                        if (removed) {
                            let roomName = `ASSISTANT_MASTER_ROOM`;
                            this.io.to(roomName).emit("updaterConnected", {
                                CDFILIAL: socket.CDFILIAL,
                                CONNECTED: false,
                            });
                        }
                    }
                });

                //   socket.removeAllListeners('execMonitoring');
                MonitoringSocket.registerEvents(socket);
                UpdaterSocket.registerEvents(socket);

                this.lastSocketId = socket.id;
            } catch (error) {
                console.log(`ERR: ${error.message}`);
            }
        });
    }

    getSocketHeaderInfo(socket) {
        let { headers } = socket.handshake;
        const defaults = {
            agent: "",
            cdfilial: "",
            hostname: "",
            version: "",
            "user-agent": "",
            "x-real-ip": socket.handshake.address,
        };
        let { agent, cdfilial, hostname, version, ...others } = {
            ...defaults,
            ...headers,
        };
        let ip = others["x-real-ip"];
        let userAgent = others["user-agent"];
        let log = [agent, cdfilial, hostname, ip]
            .filter((value) => value)
            .join(" | ");
        return { log, agent, cdfilial, hostname, version, ip, userAgent };
    }

    login(socket, data) {
        // Não é uma filial que está conectando e sim um portal
        if (!data.CDFILIAL) {
            AuthService.autenticar(data.email, data.password)
                .then((result) => {
                    socket.email = data.email;
                    this.portalSocketId = socket.id;
                    this.io.to(socket.id).emit("confirmLogin", {
                        error: false,
                        message: "Você está conectado",
                        result: result,
                    });

                    if (!data.slave && !data.updater) {
                        if (this.slavelist === undefined) {
                            this.getDashboardFiliais(data).then((dashboard) => {
                                this.io.to(socket.id).emit("login", {
                                    text: "PORTAL CONECTADO",
                                    error: false,
                                    message: dashboard,
                                    result,
                                });
                            });
                        } else {
                            this.io.to(socket.id).emit("login", {
                                text: "PORTAL CONECTADO",
                                error: false,
                                message: this.slavelist,
                                result,
                            });
                        }
                    }
                })
                .catch((error) => {
                    console.log(error);
                    this.io
                        .to(socket.id)
                        .emit("message", "Você não é um usuário autorizado!");
                    socket.disconnect();
                });
        } else {
            // Uma filial se coneta ao master (Updater ou Monitoring)
            AuthService.autenticar(data.email, "", data.CDFILIAL)
                .then((result) => {
                    socket.email = data.email;
                    socket.CDFILIAL = data.CDFILIAL;

                    // IF -> Conexão de client do Updater
                    // ELSE -> Conexão de client do Monitoring
                    if (data.updater) {
                        UpdaterSocket.addClient({
                            id: socket.id,
                            socket: socket,
                            time: new Date().getTime(),
                        });
                        this.getRoom("ASSISTANT_MASTER_ROOM").emit(
                            "updaterConnected",
                            {
                                CDFILIAL: socket.CDFILIAL,
                                CONNECTED: true,
                            }
                        );
                    } else {
                        const subsidiaryIndex = this.slavelist.findIndex(
                            (item) => {
                                return item.codigo === data.CDFILIAL;
                            }
                        );

                        if (this.slavelist[subsidiaryIndex]) {
                            this.slavelist[subsidiaryIndex].situacao = "Ativo";
                            this.slavelist[subsidiaryIndex].socketId =
                                socket.id;

                            MonitoringSocket.addClient({
                                id: socket.id,
                                socket: socket,
                                time: new Date().getTime(),
                            });

                            this.io.to(socket.id).emit("message", result);
                            this.io.to(socket.id).emit("command", {
                                type: "downloadFromUrl",
                                data: { dataUrl: "/API/getStructure" },
                            });

                            if (this.portalSocketId) {
                                this.io.to(this.portalSocketId).emit("login", {
                                    text: "UNIDADE CONECTADA",
                                    error: false,
                                    message: {
                                        data: this.slavelist,
                                    },
                                    result,
                                });
                            }
                        }
                    }
                })
                .catch((error) => {
                    console.log(error);
                    this.io
                        .to(socket.id)
                        .emit("message", "Você não é um usuário autorizado!");
                    socket.disconnect();
                });
        }
    }

    async getDashboardFiliais(data = null) {
        const filiais = [];
        const queryResultAll = await adqGetDashboardFiliais.get(
            paginacao(data)
        );
        const { rows } = queryResultAll;
        const assConfig = await this.getAssConfig();

        rows.forEach((element) => {
            const filialRow = {
                id: element[0],
                codigo: element[0],
                name: element[1],
                situacao: "Inativa",
                email: "",
                socketId: "",
                sat: element[3] == 0 ? "N" : "E", // Tratamento especial para o Sat: S => Sucesso | E => Error | N => Filial não utiliza SAT
                websocket: "",
                nrorg: element[2],
            };

            //Cria todos os monitoramentos dinâmicos na row para funcionamento do filtro | N => Erro (bolinha vermelha)
            assConfig.forEach((config) => {
                filialRow[config.ID] = "N";
            });
            filiais.push(filialRow);
        });

        if (this.slavelist !== undefined) {
            filiais.concat(this.slavelist);
        }

        return filiais;
    }

    async getAssConfig() {
        const config = [];
        const assConfig = await parametrizacoesModel.getAssConfig(true);
        assConfig.rows.forEach((element) => {
            const [
                ID,
                NOME,
                DESCRICAO,
                TIPO,
                ORDEM,
                ATIVO,
                COMANDO,
                INTERVALO,
            ] = element;
            config.push({
                ID: ID.toString(),
                NOME,
                DESCRICAO,
                TIPO,
                ORDEM,
                ATIVO,
                COMANDO,
                INTERVALO,
            });
        });
        return config;
    }
}

module.exports = new SocketService();